/*-
 * Copyright (c) 2008 Gabor Kovesdan <gabor@FreeBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/cdefs.h>

#ifndef lint
__FBSDID("$FreeBSD$");
#endif /* not lint */

#include <sys/stat.h>
#include <sys/types.h>

#include <archive.h>
#include <archive_entry.h>
#include <err.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "grep.h"

int
archive_tree(char **argv)
{
	struct file	*f;
	int		 c = 0, i;

	for(i = 0; argv[i] != NULL; i++) {
		if ((f = malloc(sizeof(f))) == NULL)
			err(2, NULL);
		if ((f->gzf = archive_read_new()) == NULL)
			err(2, NULL);
		if((f->entry = malloc(sizeof(f->entry))) == NULL)
			err(2, NULL);
		archive_read_support_compression_all(f->gzf);
		archive_read_support_format_all(f->gzf);
		archive_read_open_filename(f->gzf, argv[i], 1024);

		while (archive_read_next_header(f->gzf, &(f->entry)) != ARCHIVE_OK) {
			f->file = argv[i];
			c += procarchive(f);
		}
		free(f->gzf);
		free(f->entry);
		free(f);

		return (c);
	}
}

char *
archive_fgetln(struct archive *gzf)
{
	char	*buf;

	if ((buf = calloc(sizeof(char), 1024)) == NULL)
		err(2, NULL);

	if (archive_read_data(gzf, buf, 1024) == 0)
		return (NULL);
	else
		return (buf);
}

int
procarchive(struct file *f)
{
	struct str	 ln;
	int		 c, t, z, nottext;

/* XXX: bin check for archives
	nottext = grep_bin_file(f); */
	nottext = 0;

	if (nottext && binbehave == BIN_FILE_SKIP) {
		return (0);
	}

	ln.file = f->file;
	ln.line_no = 0;
	ln.len = 0;
	linesqueued = 0;
	tail = 0;
	ln.off = -1;

	if (Bflag > 0)
		initqueue();
	for (c = 0;  c == 0 || !(lflag || qflag); ) {
		ln.off += ln.len + 1;
		if ((ln.dat = archive_fgetln(f->gzf)) == NULL)
			break;
		ln.len = strlen(ln.dat);
		if (ln.len > 0 && ln.dat[ln.len - 1] == '\n')
			--ln.len;
		ln.line_no++;

		z = tail;

		if ((t = procline(&ln, nottext)) == 0 && Bflag > 0 && z == 0) {
			enqueue(&ln);
			linesqueued++;
		}
		c += t;
	}
	if (Bflag > 0)
		clearqueue();
	grep_close(f);

	if (cflag) {
		if (!hflag)
			printf("%s:", ln.file);
		printf("%u\n", c);
	}
	if (lflag && c != 0)
		printf("%s\n", f->file);
	if (Lflag && c == 0)
		printf("%s\n", f->file);
	if (c && !cflag && !lflag && !Lflag &&
		binbehave == BIN_FILE_BIN && nottext && !qflag)
		printf("Binary file %s matches\n", f->file);

	return (c);
}
